write_stack_trace

Function write_stack_trace 

pub fn write_stack_trace(
    stack_trace: &StackTrace<'_>,
    out_stream: &mut impl Write,
    debug_info: &mut SelfInfo,
    tty_config: TtyConfig,
    limits: &WriteStackTraceLimits,
) -> Result<(), Error>
Clone of debug.writeStackTrace, but can be configured to stop at either a frame count, or when hitting jsc LLInt Additionally, the printing function does not print the ^, instead it highlights the word at the column. This Makes each frame take up two lines instead of three.